require 'treetop'

module Aster
  
  Treetop.load(File.dirname(__FILE__) + "/sexp.treetop")

  class Parser

    def parse(text)
      lines = text.split("\n")
      parse_lines lines
    end

    private

    def text_indent(text)
      return -1 if text.nil?
      white_space = text.match(/^( +)/)
      return 0 if white_space.nil?
      size = white_space[0].size
      size % 2 == 0 ? size / 2 : -1
    end

    def raw_parse_line(line)
      @sexp ||= SexpParser.new

      result = @sexp.parse(line)

      if result && result.value
        res = result.value
        def res.garbage?
          false
        end
        res
      else
        # garbage mode
        result = line.split(' ', 2)
        def result.garbage?
          true
        end
        result
      end

    end

    def parse_line(line)
      Command.new raw_parse_line(line)
    end

    def parse_lines(lines)
      current_indent = (lines.first ? text_indent(lines.first) : 0)
      last_line = nil
      data = []
      lines.each_with_index do |line, i|
        # Skip Empty lines
        next if line.strip.empty?
        
        # Skip Comments
        next if line.strip[0] == "#"

        next_line = lines[i + 1]
        indent, next_indent = text_indent(line), text_indent(next_line)
        command = parse_line(line)

        # Grab sub lines if possible
        if next_indent > indent
          j = i + 1
          sublines = []
          while(look_ahead_line = lines[j])
            look_ahead_indent = text_indent(look_ahead_line)
            if look_ahead_indent > indent
              sublines << lines.slice!(j)
            else
              break
            end
          end

          begin
            command.sub_commands = parse_lines sublines
          rescue
            command.sub_commands = :GARBAGE
          end
          command.sub_text = sublines.map do |subline|
            subline.gsub(/^[ ]{#{next_indent*2}}/, '')
          end.join("\n")
        end
        data << command
      end
      data
    end


  end

end